/*
 *  projections.c
 *
 *  Global definitions needed. So we have them in one single place
 * 
 *  Copyright Helmut Dersch and Daniel M. German
 *  
 *  Dec 2005
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this software; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *  Author: Daniel M German dmgerman at uvic doooot ca
 * 
 */

#ifndef _PROJECTIONS_H_

enum
{                               // Enumerates for Image.format
    _rectilinear = 0,
    _panorama = 1,
    _fisheye_circ = 2,
    _fisheye_ff = 3,
    _equirectangular = 4,
    _spherical_cp = 5,
    _spherical_tp = 6,
    _mirror = 7,
    _orthographic = 8,
    _cubic = 9,
    _stereographic = 10,
    _mercator = 11,
    _trans_mercator = 12,
    _trans_panorama = 13,
    _sinusoidal = 14,
    _lambert    = 15,
    _lambertazimuthal  = 16,
    _albersequalareaconic = 17,
    _millercylindrical = 18,
    _architectural = 19,
    _peirce        = 20,
    _adamsws1      = 21,
    _adamsws2      = 22,
    _rectilinear_segmented = 23,
};

enum
{                               // Enumerates external number of panorama f<index>
    PANO_FORMAT_RECTILINEAR = 0,
    PANO_FORMAT_PANORAMA = 1,
    PANO_FORMAT_EQUIRECTANGULAR = 2,
    PANO_FORMAT_FISHEYE_FF = 3,
    PANO_FORMAT_STEREOGRAPHIC = 4,
    PANO_FORMAT_MERCATOR = 5,
    PANO_FORMAT_TRANS_MERCATOR = 6,
    PANO_FORMAT_SINUSOIDAL = 7,
    PANO_FORMAT_LAMBERT_EQUAL_AREA_CONIC = 8,
    PANO_FORMAT_LAMBERT_AZIMUTHAL = 9,
    PANO_FORMAT_ALBERS_EQUAL_AREA_CONIC = 10,
    PANO_FORMAT_MILLER_CYLINDRICAL = 11,
    PANO_FORMAT_ARCHITECTURAL = 12,
    PANO_FORMAT_PEIRCE = 13,
    PANO_FORMAT_ADAMS_WS_1= 14,
    PANO_FORMAT_ADAMS_WS_2= 15,
    PANO_FORMAT_RECTILINEAR_SEGMENTED= 16,
};

#define PANO_FORMAT_COUNT 17


// THIS IS JUST A HACK... Eventually it should be removed

#define PROJECTIONS_LABELS   {"PanoType0","Normal (rectilinear)"},\
  {"PanoType1","Cylindrical"},\
  {"PanoType2","Equirectangular"},\
  {"PanoType3","Full Frame"},\
  {"PanoType4","Stereographic"},\
  {"PanoType5","Mercator"},\
  {"PanoType6","Transverse mercator"},\
  {"PanoType7","Sinusoidal"},\
  {"PanoType8","Lambert Cylindrical Equal Area"},\
  {"PanoType9","Lambert Azimuthal Equal Area"},\
  {"PanoType10","Albers Conical Equal Area"},\
  {"PanoType11","Miller Cylindrical"},\
  {"PanoType12","Architectural"},\
  {"PanoType13","Peirce Quincuncial"},\
  {"PanoType14","Adams World in a Square I"},\
  {"PanoType15","Adams World in a Square 2"},\
  {"PanoType16","Rectilinear segmented"},

int panoProjectionFormatCount(void);

int panoProjectionFeaturesQuery(int projection, pano_projection_features *features);

extern char *panoFormatAbbr[];
extern char *panoFormatNames[];
int panoProjectionsExternalIndexByName(char *projection);
int panoProjectionVerifyDoable(Image *pano);
void panoProjectionsList(void);
int panoProjectionsInternalIndexByName(char *projection);



int rect_pano			( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
int pano_rect			( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
int pano_erect			( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
int erect_pano			( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
int sphere_cp_erect	( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
int sphere_tp_erect	( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
int erect_sphere_cp	( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
int rect_sphere_tp		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
int sphere_tp_rect		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
int sphere_cp_pano		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
int rect_erect			( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
int erect_rect			( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
int erect_sphere_tp	( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
int mirror_erect		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
int mercator_erect		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
int erect_mercator		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
int lambert_erect		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
int erect_lambert		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
int erect_lambertazimuthal( double x_dest,double  y_dest, double* x_src, double* y_src, void* params);
int lambertazimuthal_erect( double x_dest,double  y_dest, double* x_src, double* y_src, void* params);
int transmercator_erect		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
int erect_transmercator		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
int sinusoidal_erect		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
int erect_sinusoidal		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
int stereographic_erect		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
int erect_stereographic		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
int albersequalareaconic_erect	( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
int erect_albersequalareaconic	( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
int albersequalareaconic_distance	( double x, double *x_src, void* params );
int millercylindrical_erect		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
int erect_millercylindrical		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
int architectural_erect		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
int erect_architectural		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );

int erect_adamsworldinasquare1(double x_dest, double y_dest, double* x_src, double* y_src, void *params) ;
int adamsworldinasquare1_erect(double x_dest, double y_dest, double* x_src, double* y_src, void *params) ;
int erect_adamsworldinasquare2(double x_dest, double y_dest, double* x_src, double* y_src, void *params) ;
int adamsworldinasquare2_erect(double x_dest, double y_dest, double* x_src, double* y_src, void *params) ;

int erect_rectseg(double x_dest, double y_dest, double* x_src, double* y_src, void *params) ;
int rectseg(double x_dest, double y_dest, double* x_src, double* y_src, void *params) ;




#define _PROJECTIONS_H_

#endif

